@@ -18,6 +18,10 @@
 #undef _FILE_OFFSET_BITS
 #define _STRUCTURED_PROC 1
 
+// fix compilation issue on SunOS 5.10, see:
+// https://github.com/giampaolo/psutil/issues/421
+#define NEW_MIB_COMPLIANT
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/proc.h>
@@ -452,29 +456,24 @@ psutil_per_cpu_times(PyObject *self, PyObject *args)
         goto error;
     }
 
-    numcpus = sysconf(_SC_NPROCESSORS_ONLN) - 1;
-    for (i = 0; i <= numcpus; i++) {
-        ksp = kstat_lookup(kc, "cpu_stat", i, NULL);
-        if (ksp == NULL) {
-            PyErr_SetFromErrno(PyExc_OSError);
-            goto error;
+    for (ksp = kc->kc_chain; ksp != NULL; ksp = ksp->ks_next) {
+        if (strcmp(ksp->ks_module, "cpu_stat") == 0) {
+            if (kstat_read(kc, ksp, &cs) == -1) {
+                PyErr_SetFromErrno(PyExc_OSError);
+                goto error;
+            }
+            py_cputime = Py_BuildValue("ffff",
+                                       (float)cs.cpu_sysinfo.cpu[CPU_USER],
+                                       (float)cs.cpu_sysinfo.cpu[CPU_KERNEL],
+                                       (float)cs.cpu_sysinfo.cpu[CPU_IDLE],
+                                       (float)cs.cpu_sysinfo.cpu[CPU_WAIT]);
+            if (py_cputime == NULL)
+                goto error;
+            if (PyList_Append(py_retlist, py_cputime))
+                goto error;
+            Py_DECREF(py_cputime);
+            py_cputime = NULL;
         }
-        if (kstat_read(kc, ksp, &cs) == -1) {
-            PyErr_SetFromErrno(PyExc_OSError);
-            goto error;
-        }
-
-        py_cputime = Py_BuildValue("ffff",
-                                   (float)cs.cpu_sysinfo.cpu[CPU_USER],
-                                   (float)cs.cpu_sysinfo.cpu[CPU_KERNEL],
-                                   (float)cs.cpu_sysinfo.cpu[CPU_IDLE],
-                                   (float)cs.cpu_sysinfo.cpu[CPU_WAIT]);
-        if (py_cputime == NULL)
-            goto error;
-        if (PyList_Append(py_retlist, py_cputime))
-            goto error;
-        Py_DECREF(py_cputime);
-
     }
 
     kstat_close(kc);
@@ -793,6 +792,7 @@ static int PSUTIL_CONN_NONE = 128;
 
 /*
  * Return TCP and UDP connections opened by process.
+ * UNIX sockets are excluded.
  *
  * Thanks to:
  * https://github.com/DavidGriffith/finx/blob/master/
@@ -802,7 +802,7 @@ static int PSUTIL_CONN_NONE = 128;
  *     cmd-inet/usr.bin/netstat/netstat.c
  */
 static PyObject *
-psutil_proc_connections(PyObject *self, PyObject *args)
+psutil_net_connections(PyObject *self, PyObject *args)
 {
     long pid;
     int sd = NULL;
@@ -816,6 +816,7 @@ psutil_proc_connections(PyObject *self, PyObject *args)
     int i, flags, getcode, num_ent, state;
     char lip[200], rip[200];
     int lport, rport;
+    int processed_pid;
     struct strbuf ctlbuf, databuf;
     struct T_optmgmt_req *tor = (struct T_optmgmt_req *)buf;
     struct T_optmgmt_ack *toa = (struct T_optmgmt_ack *)buf;
@@ -929,8 +930,8 @@ psutil_proc_connections(PyObject *self, PyObject *args)
             tp = (mib2_tcpConnEntry_t *)databuf.buf;
             num_ent = mibhdr->len / sizeof(mib2_tcpConnEntry_t);
             for (i = 0; i < num_ent; i++, tp++) {
-                // check PID
-                if (tp->tcpConnCreationProcess != pid)
+                processed_pid = tp->tcpConnCreationProcess;
+                if (pid != -1 && processed_pid != pid)
                     continue;
                 // construct local/remote addresses
                 inet_ntop(AF_INET, &tp->tcpConnLocalAddress, lip, sizeof(lip));
@@ -953,8 +954,9 @@ psutil_proc_connections(PyObject *self, PyObject *args)
                 state = tp->tcpConnEntryInfo.ce_state;
 
                 // add item
-                py_tuple = Py_BuildValue("(iiiNNi)", -1, AF_INET, SOCK_STREAM,
-                                         py_laddr, py_raddr, state);
+                py_tuple = Py_BuildValue("(iiiNNiI)", -1, AF_INET, SOCK_STREAM,
+                                         py_laddr, py_raddr, state,
+                                         processed_pid);
                 if (!py_tuple) {
                     goto error;
                 }
@@ -971,14 +973,12 @@ psutil_proc_connections(PyObject *self, PyObject *args)
             num_ent = mibhdr->len / sizeof(mib2_tcp6ConnEntry_t);
 
             for (i = 0; i < num_ent; i++, tp6++) {
-                // check PID
-                if (tp6->tcp6ConnCreationProcess != pid)
+                processed_pid = tp6->tcp6ConnCreationProcess;
+                if (pid != -1 && processed_pid != pid)
                     continue;
                 // construct local/remote addresses
-                inet_ntop(AF_INET6, &tp6->tcp6ConnLocalAddress, lip,
-                          sizeof(lip));
-                inet_ntop(AF_INET6, &tp6->tcp6ConnRemAddress, rip,
-                          sizeof(rip));
+                inet_ntop(AF_INET6, &tp6->tcp6ConnLocalAddress, lip, sizeof(lip));
+                inet_ntop(AF_INET6, &tp6->tcp6ConnRemAddress, rip, sizeof(rip));
                 lport = tp6->tcp6ConnLocalPort;
                 rport = tp6->tcp6ConnRemPort;
 
@@ -997,9 +997,8 @@ psutil_proc_connections(PyObject *self, PyObject *args)
                 state = tp6->tcp6ConnEntryInfo.ce_state;
 
                 // add item
-                py_tuple = Py_BuildValue("(iiiNNi)", -1, AF_INET6, SOCK_STREAM,
-                                         py_laddr, py_raddr, state);
-
+                py_tuple = Py_BuildValue("(iiiNNiI)", -1, AF_INET6, SOCK_STREAM,
+                                         py_laddr, py_raddr, state, processed_pid);
                 if (!py_tuple) {
                     goto error;
                 }
@@ -1009,13 +1008,20 @@ psutil_proc_connections(PyObject *self, PyObject *args)
             }
         }
 #endif
-        else if (mibhdr->level == MIB2_UDP || mibhdr->level == MIB2_UDP_ENTRY)
-        {
+        // UDPv4
+        else if (mibhdr->level == MIB2_UDP || mibhdr->level == MIB2_UDP_ENTRY) {
             ude = (mib2_udpEntry_t *)databuf.buf;
             num_ent = mibhdr->len / sizeof(mib2_udpEntry_t);
             for (i = 0; i < num_ent; i++, ude++) {
-                // check PID
-                if (ude->udpCreationProcess != pid)
+                processed_pid = ude->udpCreationProcess;
+                if (pid != -1 && processed_pid != pid)
+                    continue;
+                // XXX Very ugly hack! It seems we get here only the first
+                // time we bump into a UDPv4 socket.  PID is a very high
+                // number (clearly impossible) and the address does not
+                // belong to any valid interface.  Not sure what else
+                // to do other than skipping.
+                if (processed_pid > 131072)
                     continue;
                 inet_ntop(AF_INET, &ude->udpLocalAddress, lip, sizeof(lip));
                 lport = ude->udpLocalPort;
@@ -1025,8 +1031,9 @@ psutil_proc_connections(PyObject *self, PyObject *args)
                 py_raddr = Py_BuildValue("()");
                 if (!py_raddr)
                     goto error;
-                py_tuple = Py_BuildValue("(iiiNNi)", -1, AF_INET, SOCK_DGRAM,
-                                         py_laddr, py_raddr, PSUTIL_CONN_NONE);
+                py_tuple = Py_BuildValue("(iiiNNiI)", -1, AF_INET, SOCK_DGRAM,
+                                         py_laddr, py_raddr, PSUTIL_CONN_NONE,
+                                         processed_pid);
                 if (!py_tuple) {
                     goto error;
                 }
@@ -1036,13 +1043,13 @@ psutil_proc_connections(PyObject *self, PyObject *args)
             }
         }
 #if defined(AF_INET6)
-        else if (mibhdr->level == MIB2_UDP6 ||
-                    mibhdr->level == MIB2_UDP6_ENTRY) {
+        // UDPv6
+        else if (mibhdr->level == MIB2_UDP6 || mibhdr->level == MIB2_UDP6_ENTRY) {
             ude6 = (mib2_udp6Entry_t *)databuf.buf;
             num_ent = mibhdr->len / sizeof(mib2_udp6Entry_t);
             for (i = 0; i < num_ent; i++, ude6++) {
-                // check PID
-                if (ude6->udp6CreationProcess != pid)
+                processed_pid = ude6->udp6CreationProcess;
+                if (pid != -1 && processed_pid != pid)
                     continue;
                 inet_ntop(AF_INET6, &ude6->udp6LocalAddress, lip, sizeof(lip));
                 lport = ude6->udp6LocalPort;
@@ -1052,8 +1059,9 @@ psutil_proc_connections(PyObject *self, PyObject *args)
                 py_raddr = Py_BuildValue("()");
                 if (!py_raddr)
                     goto error;
-                py_tuple = Py_BuildValue("(iiiNNi)", -1, AF_INET6, SOCK_DGRAM,
-                                         py_laddr, py_raddr, PSUTIL_CONN_NONE);
+                py_tuple = Py_BuildValue("(iiiNNiI)", -1, AF_INET6, SOCK_DGRAM,
+                                         py_laddr, py_raddr, PSUTIL_CONN_NONE,
+                                         processed_pid);
                 if (!py_tuple) {
                     goto error;
                 }
@@ -1139,8 +1147,7 @@ error:
     // mimic os.cpu_count()
     if (kc != NULL)
         kstat_close(kc);
-    Py_INCREF(Py_None);
-    return Py_None;
+    Py_RETURN_NONE;
 }
 
 
@@ -1165,8 +1172,6 @@ PsutilMethods[] =
      "Return process memory mappings"},
     {"proc_num_ctx_switches", psutil_proc_num_ctx_switches, METH_VARARGS,
      "Return the number of context switches performed by process"},
-    {"proc_connections", psutil_proc_connections, METH_VARARGS,
-     "Return TCP and UDP connections opened by process."},
 
     // --- system-related functions
     {"swap_mem", psutil_swap_mem, METH_VARARGS,
@@ -1185,8 +1190,10 @@ PsutilMethods[] =
      "Return system boot time in seconds since the EPOCH."},
     {"cpu_count_phys", psutil_cpu_count_phys, METH_VARARGS,
      "Return the number of physical CPUs on the system."},
+    {"net_connections", psutil_net_connections, METH_VARARGS,
+     "Return TCP and UDP syste-wide open connections."},
 
-    {NULL, NULL, 0, NULL}
+{NULL, NULL, 0, NULL}
 };
 
 
@@ -1241,6 +1248,8 @@ void init_psutil_sunos(void)
 #else
     PyObject *module = Py_InitModule("_psutil_sunos", PsutilMethods);
 #endif
+    PyModule_AddIntConstant(module, "version", PSUTIL_VERSION);
+
     PyModule_AddIntConstant(module, "SSLEEP", SSLEEP);
     PyModule_AddIntConstant(module, "SRUN", SRUN);
     PyModule_AddIntConstant(module, "SZOMB", SZOMB);
